/*
 * nfc_ndef_decoder.c
 *
 * Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
 * ALL RIGHTS RESERVED
 */

/**********************************************************************************************************************
 * Header Files
 *********************************************************************************************************************/
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <xdc/runtime/System.h>
#include "nfc_ndef_decoder.h"
#include "driverlib/debug.h"

/*********************************************************************************************************************
 * Preprocessor #define Constants
 *********************************************************************************************************************/
#define WIFI_LABEL_LENGTH 7
#define MAX_SSID_LENGTH 32
#define MAX_SECURITY_TYPE_LENGTH 4

/*********************************************************************************************************************
 *  Function Definitions
 *********************************************************************************************************************/

/******************************************************************************
 *
 *! Decodes NFC Message meta-data and payload information.
 *!
 *! \param psNDEFDataDecoded is a pointer to the sNDEFMessageData structure to
 *! be filled.
 *! \param pui8Buffer is a pointer to the raw NFC data buffer from which to
 *! decode the data.
 *! \param ui16BufferMaxLength is the maximum number of bytes the buffer
 *! can hold. This parameter is used to prevent reading past the end of the
 *! buffer.
 *!
 *! This function takes in a buffer of raw NFC data and fills up an
 *! sNDEFMessageData structure. This function is the first step to decoding an
 *! NFC Message. The next step is to decode the Message Payload, which is the record.
 *! The decoded sNDEFMessageData structure has a field named \b pui8Type. The
 *! \b pui8Type field defines the record type and therefore indicates which
 *! RecordDecoder function to use on the Message Payload.
 *!
 *! \return This function returns \b STATUS_SUCCESS (1) or \b STATUS_FAIL (0).
 *
 * Note: local variables are used to break out fields from the header for
 *       clarity. ui32HeaderSize is used to keep track of how large the header
 *       is in bytes. It is used to computer the size of the payload at the end.
 *       (length of Buffer - HeaderSize = Payload length)
 *
 ******************************************************************************/
bool
NFC_NDEFMessageDecoder(sNDEFMessageData *psNDEFDataDecoded,
                       uint8_t *pui8Buffer,
                       uint16_t ui16BufferMaxLength)
{
    sNDEFMessageData *psMessage;
    uint8_t ui8StatusByte,ui8TypeLength,ui8IDLength;
    uint8_t *pui8PayloadPtr;
    uint32_t ui32HeaderSize = 0;
    uint32_t ui32PayloadLength=0;
    uint32_t x;

    /* Check Input for Validity */
    ASSERT(pui8Buffer != 0);
    ASSERT(ui16BufferMaxLength > 0);

    /* Minimum length of header is 5 bytes. */
    if(ui16BufferMaxLength <= 5)
    {
        System_printf("ERR: NDEFMessageDecoder: Invalid Input\n");
        System_flush();
        return STATUS_FAIL;
    }

    psMessage = psNDEFDataDecoded;

    /* Load Status Byte into NDEF Structure. */
    ui8StatusByte = pui8Buffer[ui32HeaderSize];
    psMessage->sStatusByte.MB = NDEF_STATUSBYTE_GET_MB(ui8StatusByte);
    psMessage->sStatusByte.ME = NDEF_STATUSBYTE_GET_ME(ui8StatusByte);
    psMessage->sStatusByte.CF = NDEF_STATUSBYTE_GET_CF(ui8StatusByte);
    psMessage->sStatusByte.SR = NDEF_STATUSBYTE_GET_SR(ui8StatusByte);
    psMessage->sStatusByte.IL = NDEF_STATUSBYTE_GET_IL(ui8StatusByte);
    psMessage->sStatusByte.TNF = NDEF_STATUSBYTE_GET_TNF(ui8StatusByte);

    /* Increment size of header (+1 for the size of the Status Byte). */
    ui32HeaderSize++;

    /* Load TypeLength byte into NDEF Structure. */
    ui8TypeLength = pui8Buffer[ui32HeaderSize];
    psMessage->ui8TypeLength = ui8TypeLength;

    /* Increment size of header (+1 for the size of the Status Byte). */
    ui32HeaderSize++;

    /* Determine the payload size based upon the SR field in the header. */
    switch (psMessage->sStatusByte.SR)
    {
        /* Short Record (PAYLOAD_LENGTH field is 1 byte). */
        case NDEF_STATUSBYTE_SR_1BYTEPAYLOADSIZE:
        {
            ui32PayloadLength = pui8Buffer[ui32HeaderSize];

            /* Validate Data */
            if((ui32HeaderSize + ui32PayloadLength) > ui16BufferMaxLength)
            {
                ASSERT(0);
                System_printf(
                    "ERR: NFCP2P_NDEFMessageDecoder: ui32PayloadLength > ui16BufferMaxLength\n");
                System_printf("\tYou Need a bigger buffer to hold this message.\n");
                System_flush();
                return STATUS_FAIL;
            }
            else
            {
                /* Set Payload Length */
                psMessage->ui32PayloadLength = ui32PayloadLength;
                ui32HeaderSize++;
            }
            break;
        }

        /* Normal Record (PAYLOAD_LENGTH field is 4 bytes). */
        case NDEF_STATUSBYTE_SR_4BYTEPAYLOADSIZE:
        {
            ui32PayloadLength =
                (
                    (pui8Buffer[ui32HeaderSize + 3] << 0*8) |
                    (pui8Buffer[ui32HeaderSize + 2] << 1*8) |
                    (pui8Buffer[ui32HeaderSize + 1] << 2*8) |
                    (pui8Buffer[ui32HeaderSize + 0] << 3*8)
                );

            /* Validate Data */
            if((ui32HeaderSize + ui32PayloadLength) > ui16BufferMaxLength)
            {
                ASSERT(0);
                System_printf(
                    "ERR: NFCP2P_NDEFMessageDecoder: ui32PayloadLength > ui16BufferMaxLength\n");
                System_printf("\tYou Need a bigger buffer to hold this message.\n");
                System_flush();
                return STATUS_FAIL;
            }
            else
            {
                /* Set Payload Length */
                psMessage->ui32PayloadLength = ui32PayloadLength;
                ui32HeaderSize = ui32HeaderSize + 4;
            }
            break;
        }

        /* This should never happen. return error. */
        default:
        {
            System_printf("NDEFMessageDecoder: ERR decoding SR bit \n");
            System_flush();
            ASSERT(0);
            return STATUS_FAIL;
        }
    }

    /* Load ID_LENGTH field, if it exists. Depends on StatusByte.IL. */
    switch (psMessage->sStatusByte.IL)
    {
        /* ID_LENGTH field exists. Load it to the NDEF structure. */
        case NDEF_STATUSBYTE_IL_IDLENGTHPRESENT:
        {
            ui8IDLength = pui8Buffer[ui32HeaderSize];
            psMessage->ui8IDLength = ui8IDLength;
            ui32HeaderSize++;
            break;
        }

        /*
         * ID_LENGTH field does not exist and thus the ID field doesnt exists.
         * Load 0 to NDEF structure to express this
         */
        case NDEF_STATUSBYTE_IL_IDLENGTHABSENT:
        {
            psMessage->ui8IDLength = 0;
            break;
        }

        /* This should never happen. return error. */
        default:
        {
            System_printf(
                "ERR: Invalid ID_LENGTH field Detected in NDEFMessageDecoder\n");
            System_flush();
            ASSERT(0);
            return STATUS_FAIL;
        }
    }

    /*
     * Load TYPE field based on length in TYPE_LENGTH field
     *
     * If TYPE_LENGTH value is larger than NDEF_TYPE_MAXSIZE truncate to MAXSIZE
     * and adjust index in buffer to end of TYPE so as to not lose data / skew
     * pointer.
     */
    if(psMessage->ui8TypeLength > NDEF_TYPE_MAXSIZE)
    {
#ifdef DEBUG_PRINT
        ASSERT(0);
        UARTprintf("ERR: MessageDecode: TYPE > NDEF_TYPE_MAXSIZE, truncating to %d bytes\n",
                   NDEF_TYPE_MAXSIZE);
        UARTprintf("    Orig Type = ");
        for(x = 0; x < psMessage->ui8TypeLength; x++)
        {
            UARTprintf("%c",pui8Buffer[ui32HeaderSize + x]);
        }
        UARTprintf("\n");
#endif

        /* Copy across truncated TYPE */
        for(x = 0; x < NDEF_TYPE_MAXSIZE; x++)
        {
            psMessage->pui8Type[x] = pui8Buffer[ui32HeaderSize];
            ui32HeaderSize++;
        }

        /* Adjust index appropriately. */
        ui32HeaderSize = ui32HeaderSize +
                         (psMessage->ui8TypeLength - NDEF_TYPE_MAXSIZE);
        psMessage->ui8TypeLength = NDEF_TYPE_MAXSIZE;
    }
    else
    {
        /*
         * No problem
         * Load Type field into NDEF structure
         */
        for(x = 0; x < psMessage->ui8TypeLength; x++)
        {
            psMessage->pui8Type[x] = pui8Buffer[ui32HeaderSize];
            ui32HeaderSize++;
        }
    }

    /*
     * Load ID field into NDEF structure. Depends on length in ID_LENGTH field.
     * if ID field is > NDEF_ID_MAXSIZE truncate to MAXSIZE
     */
    if(psMessage->ui8IDLength > NDEF_ID_MAXSIZE)
    {
#ifdef DEBUG_PRINT
        ASSERT(0);
        UARTprintf("ERR: ID_LENGTH > NDEF_ID_MAXSIZE, trucating to %d bytes\n",
                   NDEF_ID_MAXSIZE);
        UARTprintf("    Orig ID = ");
        for(x = 0; x < psMessage->ui8IDLength; x++)
        {
            UARTprintf("%c",pui8Buffer[ui32HeaderSize + x]);
        }
        UARTprintf("\n");
#endif

        /* Copy across truncated ID */
        for(x = 0; x < NDEF_ID_MAXSIZE; x++)
        {
            psMessage->pui8ID[x] = pui8Buffer[ui32HeaderSize];
            ui32HeaderSize++;
        }

        /* adjust index appropriately */
        ui32HeaderSize = ui32HeaderSize + (psMessage->ui8IDLength -
                                           NDEF_ID_MAXSIZE);
        psMessage->ui8IDLength = NDEF_ID_MAXSIZE;
    }
    else
    {
        /*
         * No problem
         * Load ID field into NDEF structure
         */
        for(x = 0; x < psMessage->ui8IDLength; x++)
        {
            psMessage->pui8ID[x] = pui8Buffer[ui32HeaderSize];
            ui32HeaderSize++;
        }
    }

    /*
     * Error Check
     * Check to make sure we didnt overrun the buffer / read beyond its bounds.
     */
    if((ui32HeaderSize + psMessage->ui32PayloadLength) > ui16BufferMaxLength)
    {
        ASSERT(0);
        System_printf("ERR: NDEFMessageDecode: Buffer OverRun / OverRead\n");
        System_flush();

        /*
         * Clear all data out of datastrucutre, dont return invalid data.
         */
        psMessage->ui32PayloadLength=0;
        psMessage->pui8PayloadPtr=0;

        return STATUS_FAIL;
    }

    /*
     * Calculate Payload Pointer (payload is located after the header)
     */
    pui8PayloadPtr = pui8Buffer + ui32HeaderSize;

    /*
     * Set the Message Payload Pointer
     */
    psMessage->pui8PayloadPtr = pui8PayloadPtr;

    return STATUS_SUCCESS;
}

/******************************************************************************
 *
 *! Decode NDEF URI Records.
 *!
 *! \param sURIRecord is a pointer to the URIRecord structure into which to
 *! decode the data.
 *! \param pui8Buffer is a pointer to the raw NFC data buffer to be decoded.
 *! \param ui32BufferLength is the length of the raw NFC data buffer.
 *!
 *! This function takes a raw NFC data buffer and decodes the data into a URI
 *! record data structure. It is assumed that the raw data buffer contains a
 *! URI record.
 *!
 *! \return This function returns \b STATUS_SUCCESS (1) or \b STATUS_FAIL (0).
 *
 ******************************************************************************/
bool
NFC_NDEFURIRecordDecoder(sNDEFURIRecord *sURIRecord,
                         uint8_t *pui8Buffer,
                         uint32_t ui32BufferLength)
{
    uint32_t ui32RecordIndex = 0;

    /* Validate Input */
    ASSERT(pui8Buffer != 0);
    ASSERT(sURIRecord != 0);
    if(
        (pui8Buffer == 0)           ||
        (sURIRecord == 0)
    )
    {
        System_printf("ERR: URIRecordDecoder: Invalid Input\n");
        System_flush();
        return STATUS_FAIL;
    }

    /*
     * Load eIDCode field into struct
     * error check that the ID code is valid.
     */
    if(pui8Buffer[ui32RecordIndex] >= NDEF_URIRECORD_IDCODE_RFU)
    {
        /*
         * IDCode not recognized, skip it.
         * (can add codes in nfc_p2p.h eNDEF_URIRecord_IDCode enumeration)
         */
        System_printf("ERR: URI Record Decoder: URI ID Code Not Recognized: 0x%x\n"
                      ,pui8Buffer[ui32RecordIndex]);
        System_flush();
        sURIRecord->eIDCode = RFU;
        ui32RecordIndex++;
        return STATUS_FAIL;
    }
    else
    {
        /* ID Code is Valid, set it. */
        sURIRecord->eIDCode = (eNDEF_URIRecord_IDCode)pui8Buffer[ui32RecordIndex];
        ui32RecordIndex++;
    }

    /* Load UTF8 String Pointer into struct */
    sURIRecord->puiUTF8String = pui8Buffer + ui32RecordIndex;

    /* Load URI string Length into struct */
    sURIRecord->ui32URILength = ui32BufferLength-ui32RecordIndex;

    return STATUS_SUCCESS;
}

/******************************************************************************
 *
 *! Decode NDEF Text Records.
 *!
 *! \param psTextDataDecoded is a pointer to the TextRecord structure to decode
 *!  the data into.
 *! \param pui8Buffer is a pointer to the raw NFC data buffer to be decoded.
 *! \param ui32BufferLength is the length of the raw NFC data buffer.
 *!
 *! This function takes a raw NFC data buffer and decodes the data into a Text
 *! record data structure. It is assumed that the raw data buffer contains a
 *! text record.
 *!
 *! \return This function returns \b STATUS_SUCCESS (1) or \b STATUS_FAIL (0).
 *
 ******************************************************************************/
bool
NFC_NDEFTextRecordDecoder(sNDEFTextRecord *psTextDataDecoded,
                          uint8_t *pui8Buffer,
                          uint32_t ui32BufferLength)
{
    sNDEFTextRecord *psTextRecord;
    uint8_t ui8StatusByte, ui8LengthLangCode, x = 0;
    uint32_t ui32RecordIndex = 0;

    /* Validate Input */
    ASSERT(pui8Buffer != 0);
    ASSERT(psTextDataDecoded != 0);
    if(
        (pui8Buffer == 0)           ||
        (psTextDataDecoded == 0)
    )
    {
        System_printf("ERR: TextRecordDecoder: Invalid Input\n");
        return STATUS_FAIL;
    }

    /*  Initialize (done to insure 0 as sentinel in language Code) */
    psTextRecord = psTextDataDecoded;
    for(x = 0; x < NDEF_TEXTRECORD_LANGUAGECODE_MAXSIZE; x++)
    {
        psTextRecord->pui8LanguageCode[x] = 0;
    }
    psTextRecord->ui32TextLength = 0;

    /*  Load STATUSBYTE field */
    ui8StatusByte = pui8Buffer[ui32RecordIndex];
    psTextRecord->sStatusByte.bUTFcode =
        NDEF_TEXTRECORD_STATUSBYTE_GET_UTF(ui8StatusByte);
    psTextRecord->sStatusByte.bRFU =
        NDEF_TEXTRECORD_STATUSBYTE_GET_RFU(ui8StatusByte);
    ui8LengthLangCode =
        NDEF_TEXTRECORD_STATUSBYTE_GET_LENGTHLANGCODE(ui8StatusByte);
    psTextRecord->sStatusByte.ui5LengthLangCode = ui8LengthLangCode;
    ui32RecordIndex++;

    /*
     * The StatusByte.RFU should always be 0, if this is not the case return
     *  failure
     */

    if(psTextRecord->sStatusByte.bRFU != 0)
    {
        ASSERT(0);
        System_printf("Err: NDEF TextRecord Decoder: StatusByte.RFU !=0\n");
        return STATUS_FAIL;
    }

    /*  LengthLangCode must be > 0 */
    if(ui8LengthLangCode <= 0)
    {
        ASSERT(0);
        System_printf("ERR: NDEFTextRecordDecoder: LengthLangCode <= 0\n");
        return STATUS_FAIL;

    }

    /*  Load LANGUAGE_CODE field */
    for(x = 0; x < ui8LengthLangCode; x++)
    {
        /*
         * If space left in LanguageCode field put character in, otherwise
         * truncate. (dont copy across, but do incriment through raw buffer)
         */
        if(x < NDEF_TEXTRECORD_LANGUAGECODE_MAXSIZE)
        {
            psTextRecord->pui8LanguageCode[x] = pui8Buffer[ui32RecordIndex];
            ui32RecordIndex++;
        }
        else
        {
            ui32RecordIndex++;
        }
    }

    /*  Validate Data */
    if(ui8LengthLangCode > NDEF_TEXTRECORD_LANGUAGECODE_MAXSIZE)
    {
        System_printf("ERR: TextRecordDecoder: LengthLangCode > ");
        System_printf("NDEF_TEXTRECORD_LANGUAGECODE_MAXSIZE, truncating %d to %d",
                      ui8LengthLangCode,NDEF_TEXTRECORD_LANGUAGECODE_MAXSIZE);
        System_printf("\n");
        psTextRecord->sStatusByte.ui5LengthLangCode =
            NDEF_TEXTRECORD_LANGUAGECODE_MAXSIZE;
    }

    /*  Load pointer to Text */
    psTextRecord->pui8Text = pui8Buffer + ui32RecordIndex;

    /*  Validate Data - make sure we dont overrun the buffer */
    if(ui32RecordIndex > ui32BufferLength)
    {
        ASSERT(0);
        System_printf("ERR: TextRecordDecoder: Text Length longer than payload.");
        System_printf("\n");
        return STATUS_FAIL;
    }
    else
    {
        /*
         * Calculate Length of Text
         * Length of text =  Length of Record - RecordIndex to this point.
         */
        psTextRecord->ui32TextLength = ui32BufferLength-ui32RecordIndex;
    }

    return STATUS_SUCCESS;
}

/******************************************************************************
 *
 *! Decode NDEF WIFI Pairing Records.
 *!
 *! \param sURIRecord is a pointer to the URIRecord structure into which to
 *! decode the data.
 *! \param pui8Buffer is a pointer to the raw NFC data buffer to be decoded.
 *! \param ui32BufferLength is the length of the raw NFC data buffer.
 *!
 *! This function takes a raw NFC data buffer and decodes the data into a URI
 *! record data structure. It is assumed that the raw data buffer contains a
 *! URI record.
 *!
 *! \return This function returns \b STATUS_SUCCESS (1) or \b STATUS_FAIL (0).
 *
 ******************************************************************************/
bool
NFC_NDEFWIFIParingDecoder(sNDEFWIFIRecord *sWIFIRecord,
                          sNDEFURIRecord *sURIRecord)
{
    uint8_t i;
    uint8_t start_index;
    char substring[100];

    /* Validate Input */
    ASSERT(sWIFIRecord != 0);
    ASSERT(sURIRecord != 0);
    if(
        (sWIFIRecord == 0)           ||
        (sURIRecord == 0)
    )
    {
        System_printf("ERR: WIFIParingDecoder: Invalid Input\n");
        System_flush();
        return STATUS_FAIL;
    }

    /* Look for Wi-Fi label */
    for(i=0; i < WIFI_LABEL_LENGTH; i++)
    {
        substring[i] = sURIRecord->puiUTF8String[i];
    }
    /* Insert null character to terminate the string */
    substring[i] = '\0';

    /* Check for Wi-Fi label */
    if(0 == strcmp("wifi://",substring))
    {
        start_index = i;

        while('/' != sURIRecord->puiUTF8String[i])
        {
            if(MAX_SSID_LENGTH <= i)
            {
                /* Invalid SSID */
                return STATUS_FAIL;
            }
            else
            {
                sWIFIRecord->ui8SSID[i-WIFI_LABEL_LENGTH] = sURIRecord->puiUTF8String[i];
                i++;
            }
        }

        /* Insert null character to terminate the string */
        sWIFIRecord->ui8SSID[i] = '\0';

        sWIFIRecord->ui8SSIDLength = i - start_index;

        /* Increase index to the next element */
        i++;

        /* Store the start index for the next section */
        start_index = i;

        /* Look for Security Type label */
        while('/' != sURIRecord->puiUTF8String[i] && MAX_SECURITY_TYPE_LENGTH > (i-start_index))
        {
            substring[i-start_index] = sURIRecord->puiUTF8String[i];
            i++;
        }
        /* Insert null character to terminate the string */
        substring[i-start_index] = '\0';

        /* Check for Security Type label */
        if(0 == strcmp("open",substring))
        {
            sWIFIRecord->eSecurityType = SECURITY_OPEN;

            /* No need to check for Security Key, clear values and return */
            i++;

            while(i < sURIRecord->ui32URILength)
            {
                sWIFIRecord->ui8SecurityKey[i-start_index] = 0;
                i++;
            }

            sWIFIRecord->ui8SecurityKeyLength = 0;

            return STATUS_SUCCESS;
        }
        else if(0 == strcmp("wep",substring) || 0 == strcmp("WEP",substring))
        {
            sWIFIRecord->eSecurityType = SECURITY_WEP;
        }
        else if(0 == strcmp("wpa",substring) || 0 == strcmp("WPA",substring))
        {
            sWIFIRecord->eSecurityType = SECURITY_WPA;
        }
        else
        {
            /* Invalid Security Type */
            return STATUS_FAIL;
        }

        /* Increase index to the next element */
        i++;

        /* Store the start index for the next section */
        start_index = i;

        while(i < sURIRecord->ui32URILength)
        {
            sWIFIRecord->ui8SecurityKey[i-start_index] = sURIRecord->puiUTF8String[i];
            i++;
        }

        sWIFIRecord->ui8SecurityKeyLength = i - start_index;

    }
    else
    {
        /* Do nothing. Invalid WiFi pairing NDEF */
        return STATUS_FAIL;
    }

    return STATUS_SUCCESS;
}
