/*
 * nfc_task.c
 *
 * Copyright (c) 2015 Texas Instruments Incorporated - http://www.ti.com/
 * ALL RIGHTS RESERVED
 */

/**********************************************************************************************************************
 * Header Files
 *********************************************************************************************************************/
#include <string.h>

/* XDCtools Header files */
#include <xdc/std.h>
#include <xdc/cfg/global.h>
#include <xdc/runtime/System.h>

/* BIOS Header files */
#include <ti/sysbios/BIOS.h>

/* TI-RTOS Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>

/* Example/Board Header files */
#include "Board.h"

/* NFCLink Header Files */
#include "nfc_spi.h"
#include "nfc_ce.h"
#include "nfc_controller.h"
#include "nfc_inc/nfc_proto.h"
#include "nfc_inc/nfc_externs.h"
#include "tag_header.h"

/* NDEF Decoder Header file */
#include "nfc_ndef_decoder.h"

/* Command Line Interface (UART) */
#include "cli_uart.h"

/*********************************************************************************************************************
 * Preprocessor #define Constants
 *********************************************************************************************************************/
#define WIFI_PAIR_DATA_OPEN {                                                            \
    0x00, 0x17,    /* NDEF Length 23bytes */                                             \
                                                                                         \
    /* NDEF start */                                                                     \
    0xD1,    /* NDEF Header MB=1, ME=1, CF=0, SR=1, IL=0, TNF=1 */                       \
                                                                                         \
    0x01,    /* Type Length 1 byte */                                                    \
                                                                                         \
    0x13,    /* Payload length 19bytes */                                                \
                                                                                         \
    0x55,    /* Type U (URI) */                                                          \
    /* Payload start */                                                                  \
                                                                                         \
    0x00,    /* URI Record Type : No abbreviation */                                     \
                                                                                         \
    0x77, 0x69, 0x66, 0x69, 0x3A, 0x2F, 0x2F,     /* wifi:// text */                     \
    /* SSID TIHome */                                                                    \
    0x54, 0x49, 0x48, 0x6F, 0x6D, 0x65,                                                  \
                                                                                         \
    0x2F,    /* 1st / */                                                                 \
                                                                                         \
    0x6F, 0x70, 0x65, 0x6E     /* security type open */                                  \
}

#define WIFI_PAIR_DATA_WPA {                                                             \
    0x00, 0x1F,    /* NDEF Length 31bytes */                                             \
                                                                                         \
    /* NDEF start */                                                                     \
    0xD1,    /* NDEF Header MB=1, ME=1, CF=0, SR=1, IL=0, TNF=1 */                       \
                                                                                         \
    0x01,    /* Type Length 1 byte */                                                    \
                                                                                         \
    0x1B,    /* Payload length 27bytes */                                                \
                                                                                         \
    0x55,    /* Type U (URI) */                                                          \
    /* Payload start */                                                                  \
                                                                                         \
    0x00,    /* URI Record Type : No abbreviation */                                     \
                                                                                         \
    0x77, 0x69, 0x66, 0x69, 0x3A, 0x2F, 0x2F,     /* wifi:// text */                     \
    /* SSID TIHome */                                                                    \
    0x54, 0x49, 0x48, 0x6F, 0x6D, 0x65,                                                  \
                                                                                         \
    0x2F,    /* 1st / */                                                                 \
                                                                                         \
    0x77, 0x70, 0x61,          /* security type wpa */                                   \
                                                                                         \
    0x2F,    /* 2nd / */                                                                 \
                                                                                         \
	0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38                                       \
}

#define SERVER_URL_DATA {                                                                \
    0x00, 0x15,    /* NDEF Length 21bytes */                                             \
                                                                                         \
    /* NDEF start */                                                                     \
    0xD1,    /* NDEF Header MB=1, ME=1, CF=0, SR=1, IL=0, TNF=1 */                       \
                                                                                         \
    0x01,    /* Type Length 1 byte */                                                    \
                                                                                         \
    0x11,    /* Payload length 17bytes */                                                \
                                                                                         \
    0x55,    /* Type U (URI) */                                                          \
    /* Payload start */                                                                  \
                                                                                         \
    0x01,     /* URI Record Type : http://www.  */                                       \
    /*Application Data : mysimplelink.net */                                             \
    0x6D, 0x79, 0x73, 0x69, 0x6D, 0x70, 0x6C, 0x65,                                      \
    0x6C, 0x69, 0x6E, 0x6B, 0x2E, 0x6E, 0x65, 0x74                                       \
} /* End of data */

#define WIFI_CLIENT_NDEF_LENGTH 20

/*********************************************************************************************************************
 * Type Declarations
 *********************************************************************************************************************/
typedef struct MESSAGE_DATA {
    uint8_t        message[0x80];
} MESSAGE_DATA;

typedef struct MsgObj {
    Queue_Elem	elem;
    Bool	val;            /* message value */
} MsgObj, *Msg;             /* Use Msg as pointer to MsgObj */

/*********************************************************************************************************************
 * Local Object Definitions
 *********************************************************************************************************************/
static const MESSAGE_DATA ui8NDEFPairBuffer = {
#ifdef WIFI_OPEN_SECURITY
    WIFI_PAIR_DATA_OPEN
#else
    WIFI_PAIR_DATA_WPA
#endif
};

static const MESSAGE_DATA ui8NDEFUrlBuffer = {
    SERVER_URL_DATA
};

Nfc_Info_t NFCInfo;

/* Counter for TRF7970A Timer*/
uint32_t g_ui32TimeoutMsCounter = 10;

/* NFC Transmission Buffer */
uint8_t g_ui8TxBuffer[256];

/* NFC Reception Buffer */
uint8_t g_ui8RxBuffer[256];

/*********************************************************************************************************************
 *  Function Definitions
 *********************************************************************************************************************/

/************************************************ Interrupt Handlers *************************************************/
void TRF7970A_IntHandler( void )
{
    g_ui8IrqFlag = TRUE;
    GPIO_toggle(Board_LED4);
    Semaphore_post( Semaphore_NFC );
}

void TimerNFCIntHandler( void )
{
    if( --g_ui32TimeoutMsCounter == 0 )
    {
        g_ui8TimeOutFlag = TRUE;
        /* Disable Timer Interrupt as it is no longer needed */
        Timer_stop( Timer_NFC );
        Semaphore_post( Semaphore_NFC );
    }
    else
    {
        Timer_start( Timer_NFC );
    }
}


/************************************************ Task Functions *************************************************/

/*
 *  ======== HeartBeatFxn ========
 *  Toggle the Board_LED0. The Task_sleep is determined by arg0 which
 *  is configured for the heartBeat Task instance.
 */
void HeartBeatFxn(UArg arg0, UArg arg1)
{
    while (1)
    {
        Task_sleep((UInt)arg0);
        GPIO_toggle(Board_LED3);
    }
}

/*
 *  ======== PairingTaskFxn ========
 *  Decode received NDEF message and check for validity
 *  to send back pairing information.
 */
void PairingTaskFxn(UArg arg0, UArg arg1)
{
	sNDEFMessageData Decoded_NDEF;
	sNDEFTextRecord Text_Record;

	while (1)
    {
        Semaphore_pend(Semaphore_Tag, BIOS_WAIT_FOREVER);

        /* Read NDEF data */
        T4T_readNDEF(g_ui8RxBuffer, WIFI_CLIENT_NDEF_LENGTH);
        /* Decode NDEF */
        if(STATUS_SUCCESS == NFC_NDEFMessageDecoder(&Decoded_NDEF, g_ui8RxBuffer, 40))
        {
            /* Get Text Record from NDEF*/
            if(STATUS_SUCCESS == NFC_NDEFTextRecordDecoder(&Text_Record, Decoded_NDEF.pui8PayloadPtr, Decoded_NDEF.ui32PayloadLength))
            {
                /* Check if this is a WIFI node */
                if(0 == strcmp("WIFI",(const char *)Text_Record.pui8Text))
                {
                    /* Write Wi-Fi Pairing Information into the Tag */
                	CLI_Write(" Valid Tag. Writing Wi-Fi Pairing Information\n\r");
                    System_printf( "Valid Tag. Writing Wi-Fi Pairing Information\n" );
                    System_flush();
                    T4T_writeNDEF((uint8_t *)&ui8NDEFPairBuffer, sizeof(MESSAGE_DATA));
                }
                /* Check if this is a BLE node */
                else if(0 == strcmp("BLE",(const char *)Text_Record.pui8Text))
                {
                    /* Write BLE pairing information into the Tag */
                	System_printf( "Valid Tag. BLE pairing not supported \n" );
                	System_flush();
                }
                /* Check if this is a ZigBee node */
                else if(0 == strcmp("ZNL",(const char *)Text_Record.pui8Text))
                {
                    /* Write ZigBee pairing information into the Tag */
                	System_printf( "Valid Tag. Zigbee pairing not supported \n" );
                    System_flush();
                }
                else
                {
                    /* Do nothing. Invalid node tag. */
                	CLI_Write(" Invalid node tag\n\r");
                    System_printf( "Invalid node tag\n" );
                    System_flush();
                }
            }
        }
    }
}

/*
 *  ======== NfcTaskFxn ========
 *  Setup communication with TRF7970A and handle
 *  NFC state machines.
 */
void NfcTaskFxn(UArg arg0, UArg arg1)
{
    MsgObj msg;     /* define msg using MsgObj struct created earlier */
    Msg msgp;       /* define pointer to MsgObj to use with queue put/get */
    msgp = &msg;

    SPI_Params MasterSpiParams;

/****************************************************************************
 * NFC Data & Variables
 ***************************************************************************/
    tNfcState eTempNFCState;

    tNfcState eCurrentNFCState;
    tNDEFStateMachine eCurrentT4TState;
    tNDEFStateMachine ePreviousT4TState;

    /* Peer to Peer TX Variables */
    uint32_t ui32PacketRemaining;
    uint8_t ui8TXBytes;
    uint16_t ui16TxIndex;
    uint32_t ui32PacketLength;
    uint8_t * pui8NdefPointer;
    uint8_t ui8FragmentSize;

    t_sNfcCEMode sCEMode;
    t_sNfcRWMode sRWMode;
    t_sNfcRWCommBitrate sRWBitrate;

    t_sNfcP2PMode sP2PMode;
    t_sNfcP2PCommBitrate sP2PBitrate;
    bool bSendFlag = true;
    bool bWifiClientConnected = false;
    uint8_t ui8SyncCount = 0;

/****************************************************************************
 * Initialze SPI
 ***************************************************************************/
    /* Set TRF7970A Callback */
    GPIO_setCallback( Board_TRF7970A_IRQ, TRF7970A_IntHandler );
    GPIO_enableInt( Board_TRF7970A_IRQ );

    SPI_Params_init( &MasterSpiParams );
    MasterSpiParams.bitRate = 4000000;
    MasterSpiParams.mode = SPI_MASTER;
    MasterSpiParams.frameFormat = SPI_POL0_PHA1;		/* clock low when inactive and sampled on leading (rising ) edge */
    MasterSpiParams.dataSize = 8;
    MasterSpiParams.transferMode = SPI_MODE_CALLBACK;
    MasterSpiParams.transferCallbackFxn = SPI_Callback;
    NFCInfo.hMasterSPI = SPI_open( EK_TM4C1294XL_SPI2, &MasterSpiParams );
    if( NFCInfo.hMasterSPI == NULL )
    {
        System_abort("Error initializing SPI\n");
    }
    else
    {
        System_printf("SPI initialized\n");
    }

    Semaphore_reset( Semaphore_NFC, 0 );
    System_flush();

    if(STATUS_SUCCESS == TRF79x0_idleMode())
    {
        System_printf("TRF7970A initialized\n");
    }
    else
    {
        System_abort("Error initializing TRF7970A\n");
    }
    System_flush();

    /* Initialize the NFC Controller */
    NFC_init();

    /* This function will configure all the settings for each protocol */
    NFC_configuration();

    /* Initialize Type 4 Tag RTD Message */
    T4T_CE_initNDEF();

    /* Initialize IDs for NFC-A, NFC-B and NFC-F */
    NFC_initIDs();

    /* Initialize the RW T2T, T3T, T4T and T5 state machines */
    T2T_init(g_ui8TxBuffer,256);
    T3T_init(g_ui8TxBuffer,256);
    T4T_init(g_ui8TxBuffer,256);
    T5T_init(g_ui8TxBuffer,256);

    System_printf("Tap a P2P or Tag Type 4 device\n");
    System_flush();

    while (1)
    {
        eTempNFCState = NFC_run();

        if(eTempNFCState == NFC_DATA_EXCHANGE_PROTOCOL)
        {
            if(NFC_CE_getModeStatus(&sCEMode))
            {
                NFC_emulatedT4TSniffer();
            }
            if(NFC_P2P_getModeStatus(&sP2PMode,&sP2PBitrate))
            {
                if(bSendFlag == true)
                {
                	/* Wait to send a connect request to avoid colission" */
                	if(ui8SyncCount < 3)
                    {
                        ui8SyncCount++;
                        ui32PacketRemaining = 0;
                    }
                    else
                    {
                        /* Clear Send Flag */
                        bSendFlag = false;

                        ui16TxIndex = 0x00;

                        if(!Queue_empty(Queue_Station_Conn))
                        {
                            /* Get WiFi client status */
                            msgp = Queue_get(Queue_Station_Conn);
                            bWifiClientConnected = 	msgp->val;
                        }

                        if(TRUE == bWifiClientConnected)
                        {
                            /* Total Length of the packet. */
                            ui32PacketLength = 21;
                            /* Pointer to the packet. */
                            pui8NdefPointer = (uint8_t *) (ui8NDEFUrlBuffer.message+2);
                        }
                        else
                        {
                            /* Total Length of the packet. */
#ifdef WIFI_OPEN_SECURITY
                            ui32PacketLength = 23;
#else
                            ui32PacketLength = 31;
#endif
                            /* Pointer to the packet. */
                            pui8NdefPointer = (uint8_t *) (ui8NDEFPairBuffer.message+2);
                        }

                        ui32PacketRemaining = ui32PacketLength;

                        if(ui32PacketRemaining < LLCP_MIU)
                        {
                            ui8FragmentSize = (uint8_t) ui32PacketRemaining;
                        }
                        else
                        {
                            ui8FragmentSize = LLCP_MIU;
                        }

                        ui8TXBytes = NFC_P2P_sendNdefPacket(pui8NdefPointer,true,ui8FragmentSize,ui32PacketLength);

                        if(ui8TXBytes)
                        {
                            ui32PacketRemaining = ui32PacketRemaining - (uint16_t) (ui8TXBytes);
                            ui16TxIndex = ui16TxIndex + (uint16_t) ui8TXBytes;
                        }
                    }
                }
                else if(ui32PacketRemaining > 0)
                /* Handles the case where NDEF message is greater than LLCP_MIU */
                {
                    if(ui32PacketRemaining < LLCP_MIU)
                    {
                        ui8FragmentSize = (uint8_t) ui32PacketRemaining;
                    }
                    else
                    {
                        ui8FragmentSize = LLCP_MIU;
                    }
                    ui8TXBytes = NFC_P2P_sendNdefPacket((uint8_t *) (pui8NdefPointer+ui16TxIndex),false,ui8FragmentSize,(uint32_t) ui32PacketLength);

                    if(ui8TXBytes)
                    {
                        ui32PacketRemaining = ui32PacketRemaining - (uint16_t) (ui8TXBytes);
                        ui16TxIndex = ui16TxIndex + (uint16_t) ui8TXBytes;
                    }
                }
                else
                {
                    /* Nothing to send */
                }
            }
            if(NFC_RW_getModeStatus(&sRWMode,&sRWBitrate))
            {
                if( sRWMode.bits.bNfcA == 1)
                {
                    if(NFC_A_getSAK() == 0x00)
                    {
                        /* T2T Tag State Machine */
                        T2T_stateMachine();
                    }
                    else if(NFC_A_getSAK() & 0x20)
                    {
                        /* T4T Tag State Machine */
                        T4T_stateMachine();
                    }
                    else
                    {
                        /* Do nothing */
                    }
                }
                else if(sRWMode.bits.bNfcB == 1)
                {
                    if(NFC_B_isISOCompliant())
                    {
                        /* T4T Tag State Machine */
                        T4T_stateMachine();
                        /* Get Current Status */
                        eCurrentT4TState = T4T_getState();

                        if(T4T_SELECTED_IDLE == eCurrentT4TState && T4T_READ_NDEF == ePreviousT4TState)
                        {
                            /* Read Finished */
                            System_printf( "\nTag Type 4 Detected\n" );
                            System_flush();
                            Semaphore_post(Semaphore_Tag);
                        }

                        ePreviousT4TState = eCurrentT4TState;
                    }
                }
                else if(sRWMode.bits.bNfcF == 1)
                {
                    /* T3T Tag State Machine */
                    T3T_stateMachine();
                }
                else if(sRWMode.bits.bISO15693 == 1)
                {
                    /* T5T Tag State Machine */
                    T5T_stateMachine();
                }
                else
                {
                	/* Do nothing. No mode supported. */
                }
            }
        }

        /* Update Current State if it has changed. */
        if( eCurrentNFCState != eTempNFCState )
        {
            if(eCurrentNFCState != NFC_TARGET_WAIT_FOR_ACTIVATION
                    && eCurrentNFCState != NFC_STATE_IDLE
                    && (eTempNFCState == NFC_PROTOCOL_ACTIVATION
                        || eTempNFCState == NFC_DISABLED))
            {
                eCurrentNFCState = eTempNFCState;

                /* Initialize the RW T2T, T3T, T4T and T5 state machines */
                T2T_init(g_ui8TxBuffer,256);
                T3T_init(g_ui8TxBuffer,256);
                T4T_init(g_ui8TxBuffer,256);
                T5T_init(g_ui8TxBuffer,256);

                /* Clear P2P sync variables */
                bSendFlag = true;
                ui8SyncCount = 0;

                /* Initialize IDs for NFC-A, NFC-B and NFC-F */
                NFC_initIDs();
            }
            else
            {
                eCurrentNFCState = eTempNFCState;
            }

            if(eCurrentNFCState == NFC_DATA_EXCHANGE_PROTOCOL)
            {
                eCurrentNFCState = eTempNFCState;
                ui32PacketRemaining = 0x00;
            }
        }
    }
}

/*
 *  ======== TimerSet ========
 *  Timer function for TRF7970A IRQ handler
 *  in milliseconds.
 */
void TimerSet( uint16_t ui16Timeoutms )
{
    unsigned int Key;

    if(ui16Timeoutms > 0)
    {
        g_ui32TimeoutMsCounter = ui16Timeoutms;
        g_ui8TimeOutFlag = FALSE;
        Key = Hwi_disable();
        Timer_setPeriodMicroSecs( Timer_NFC, 1000 );
        Timer_start( Timer_NFC );
        Hwi_restore( Key );
    }
    else
    {
        Key = Hwi_disable();
        Timer_stop( Timer_NFC );
        Hwi_restore( Key );
        /* Set Timeout Flag */
        g_ui8TimeOutFlag = TRUE;
    }
}
